package com.winhc.phoenix.example.job;

import com.mongodb.client.MongoCollection;
import com.winhc.phoenix.example.framework.es.EsFastScan;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.bson.Document;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.search.SearchHit;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2020/11/16 20:03
 */
@Slf4j
@Component
@AllArgsConstructor
public class EsScanJob {
    private RestHighLevelClient restHighLevelClient;
    private final MongoTemplate mongoTemplate;

    public void start() {
        String dsl = "{\n" +
                "  \"query\": {\n" +
                "    \"bool\": {\n" +
                "      \"must_not\": [\n" +
                "        {\"terms\": {\n" +
                "          \"deleted\": [\n" +
                "            \"0\",\n" +
                "             \"9\",\n" +
                "            \"1\"\n" +
                "          ]\n" +
                "        }}\n" +
                "      ]\n" +
                "    }\n" +
                "  }\n" +
                "}";
        MongoCollection<Document> person = mongoTemplate.getCollection("xjk_company_equity_info_deleted_error");
        Consumer<SearchHit[]> func = list -> {
            List<Document> li = Arrays.stream(list).map(d -> {
                String id = d.getId();
                Document document = new Document();
                document.put("_id", id);
                document.put("rowkey", id);
                document.put("ds", "20210422");
                document.put("tn", "company_equity_info");
                return document;
            }).collect(Collectors.toList());
            person.insertMany(li);
        };

        new EsFastScan(restHighLevelClient, func, "winhc_index_company_equity_info_v1", "_doc", dsl).scan();
    }

}
